@@ -1,7 +1,7 @@
 ################################################################
 # gnome.tcl - gnome interface to mmucl
 # 
-# Copyright (C) 2002 Mark Patton
+# Copyright (C) 2003-2004 Mark Patton
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,11 +21,13 @@
 package require Gnome
 package require Gdk
 package require GdkPixbuf
-package require Zvt
+package require Vte
 
 # FIXME: much of this should be refactored...
+# FIXME: add can_beep, audio and visual
+# FIXME: save term size on exit and restore
 
-# Strangely enough zvt seems to need the \r
+# Strangely enough vte seems to need the \r
 
 proc mmucl::display {ses str} {
     variable Mmucl
@@ -35,7 +37,7 @@ proc mmucl::display {ses str} {
     }
 
     set str [string map {\n \r\n} $str]
-    gtk::zvt_term feed $Mmucl(ses,$ses,term) $str [string length $str]
+    vte::terminal feed $Mmucl(ses,$ses,term) $str [string length $str]
 }
 
 proc mmucl::event {ses event args} {
@@ -89,7 +91,7 @@ proc mmucl::MCclear {ses args} {
     set syntax {}
     check clear $syntax $args
 
-    gtk::zvt_term reset $Mmucl(ses,$ses,term) 1
+    vte::terminal reset $Mmucl(ses,$ses,term) true false
 }
 
 proc mmucl::MCbell {ses args} {
@@ -213,14 +215,16 @@ proc mmucl::key_id {mod key} {
     return [string map $modmap [append id [join $mod ""] $key]]
 }
 
+# FIXME: break and continue?
+
 # handle a key press event
 # FIXME: should query terminal rows for values on scrolling.
-
+# FIXME per session?
 proc mmucl::key_event {event} {
     variable Mmucl
 
     set id [key_id [gdk::event get_keymod $event] \
-	    [gdk::event get_keyval $event]]
+		[gdk::event get_keyval $event]]
 
     set ses $Mmucl(cur)
     set Mmucl(ses,$ses,current_key) $id
@@ -249,8 +253,6 @@ proc mmucl::key_event {event} {
 	} End {
 	    gtk::adjustment set_value $Mmucl(ses,$ses,termadj) \
 		    $Mmucl(pref,scrollback)
-	} default {
-	    # FIXME: scroll to bottom
 	}
     }
 	
@@ -267,14 +269,12 @@ proc mmucl::key_eval {id} {
 	foreach group $Mmucl($x,forkey,key,$id) {
 	    set code [catch {$ses eval $Mmucl($x,val,$group,key,$id)} error]
     
-	    if {$code} {
-		if {$code == 3} {
-		    return 1
-		}
-		
+	    if {$code == 1} {
 		report $ses error "key $id: $error"
 	    }
 	}
+
+	return 1
     }
     
     return 0
@@ -900,17 +900,21 @@ proc mmucl::update_view {name} {
 proc mmucl::term_pref_update_color {arglist which} {
     variable Mmucl
 
-    lassign {w red green blue a} $arglist
+    lassign {w r g b a} $arglist
+    set color [list $r $g $b]
 
-    foreach v {red green blue} {
-	set Mmucl(pref,term,$v) [lreplace $Mmucl(pref,term,$v) $which $which \
-		[set $v]]
+    if {$which == 16} {
+	set Mmucl(pref,term,fg) $color
+    } elseif {$which == 17} {
+	set Mmucl(pref,term,bg) $color
+    } else {
+	lset Mmucl(pref,term,palette) $which $color
     }
 
     foreach ses $Mmucl(sessions) {
-	gtk::zvt_term set_color_scheme $Mmucl(ses,$ses,term) \
-		$Mmucl(pref,term,red) $Mmucl(pref,term,green) \
-		$Mmucl(pref,term,blue)
+	vte::terminal set_colors  $Mmucl(ses,$ses,term) \
+	    $Mmucl(pref,term,fg) $Mmucl(pref,term,bg) \
+	    $Mmucl(pref,term,palette)
     }
 }
 
@@ -921,44 +925,43 @@ proc mmucl::term_pref_update_font {gf check_bold} {
     set Mmucl(pref,font_name) [gnome::font_picker get_font_name $gf]
 
     foreach ses $Mmucl(sessions) {
-	gtk::zvt_term set_font $Mmucl(ses,$ses,term) $Mmucl(pref,font_name) \
-		$Mmucl(pref,use_bold) 
+	vte::terminal set_font_from_string $Mmucl(ses,$ses,term) \
+	    $Mmucl(pref,font_name)
+	vte::terminal set_allow_bold $Mmucl(ses,$ses,term) \
+	    $Mmucl(pref,use_bold)
     }
 }
 
+proc mmucl::term_set_background {term imagefile transparent saturation} {
+    if {$imagefile eq ""} {
+	vte::terminal set_background_image $term null
+    } else {
+	vte::terminal set_background_image_file $term $imagefile
+    }
 
-#/* background flag options */
-#/* shade background image.  This must be left as 1 for api compat! */
-#define ZVT_BACKGROUND_SHADED 0x01 
-#/* background can scroll, exclusive option with transparency */
-#define ZVT_BACKGROUND_SCROLL 0x02 
+    vte::terminal set_background_transparent $term $transparent
+    vte::terminal set_background_saturation $term $saturation
+}
 
-proc mmucl::term_pref_update_bgtype {gpe r_normal r_pixmap r_transparent \
-	c_scroll c_shade} {
+# Called when a background value in the term pref dialog changes
+proc mmucl::term_pref_update_bgtype {gpe r_normal r_image r_transparent sat} {
     variable Mmucl
 
-    foreach v {normal pixmap transparent} {
+    foreach v {normal image transparent} {
 	if {[g::object get [set r_$v] active]} {
 	    set Mmucl(pref,bg,type) $v
 	}
     }
 
-    gtk::widget set_sensitive $gpe [expr {$Mmucl(pref,bg,type) eq "pixmap"}]
+    gtk::widget set_sensitive $gpe [expr {$Mmucl(pref,bg,type) eq "image"}]
+    gtk::widget set_sensitive $sat [expr {$Mmucl(pref,bg,type) ne "normal"}]
 
     set file ""
     set trans 0
-    set flags 0
-
-    if {[g::object get $c_shade active]} {
-	set flags [expr {$flags | 0x1}]
-    }
-    if {[g::object get $c_scroll active]} {
-	set flags [expr {$flags | 0x2}]
-    }
 
     switch -exact $Mmucl(pref,bg,type) {
 	normal {
-	} pixmap {
+	} image {
 	    set file [gtk::entry get_text [gnome::file_entry gtk_entry $gpe]]
 	} transparent {
 	    set trans 1
@@ -967,42 +970,44 @@ proc mmucl::term_pref_update_bgtype {gpe r_normal r_pixmap r_transparent \
 	}
     }
 
-    set Mmucl(pref,bg,pixmap) $file
-    set Mmucl(pref,bg,trans) $trans
-    set Mmucl(pref,bg,flags) $flags
+    set Mmucl(pref,bg,image) $file
+    set Mmucl(pref,bg,transparent) $trans
+    set Mmucl(pref,bg,saturation) [gtk::range get_value $sat]
 
     foreach ses $Mmucl(sessions) {
-	gtk::zvt_term set_background $Mmucl(ses,$ses,term) \
-		$Mmucl(pref,bg,pixmap) $Mmucl(pref,bg,trans) \
-		$Mmucl(pref,bg,flags)
+	mmucl::term_set_background $Mmucl(ses,$ses,term) \
+	    $Mmucl(pref,bg,image) $Mmucl(pref,bg,transparent) \
+	    $Mmucl(pref,bg,saturation)
     }
 }
 
-proc mmucl::term_pref_color_gcs {gcs red green blue} {
+# gcs is palette followed by fg and bg  gnome color selector widgets
+
+proc mmucl::term_pref_color_gcs {gcs} {
     variable Mmucl
 
-    foreach gc $gcs r $red g $green b $blue {
+    set colors $Mmucl(pref,term,palette)
+    lappend colors $Mmucl(pref,term,fg) $Mmucl(pref,term,bg)
+
+    foreach gc $gcs color $colors {
+	lassign {r g b} $color
 	gnome::color_picker set_i16 $gc $r $g $b 0
     }
-
 }
 
 proc mmucl::term_pref_reset_color {gcs} {
     variable Mmucl
 
-    array set Mmucl [list \
-		pref,term,red   $Mmucl(term,red_default)   \
-		pref,term,green $Mmucl(term,green_default) \
-		pref,term,blue  $Mmucl(term,blue_default)  \
-		]
+    set Mmucl(pref,term,palette) $Mmucl(term,palette)
+    set Mmucl(pref,term,fg) $Mmucl(term,fg)
+    set Mmucl(pref,term,bg) $Mmucl(term,bg)
 
-    term_pref_color_gcs $gcs $Mmucl(pref,term,red) \
-	$Mmucl(pref,term,green) $Mmucl(pref,term,blue)
+    term_pref_color_gcs $gcs
 
     foreach ses $Mmucl(sessions) {
-	gtk::zvt_term set_color_scheme $Mmucl(ses,$ses,term) \
-	    $Mmucl(pref,term,red) $Mmucl(pref,term,green) \
-	    $Mmucl(pref,term,blue)
+	vte::terminal set_colors  $Mmucl(ses,$ses,term) \
+	    $Mmucl(pref,term,fg) $Mmucl(pref,term,bg) \
+	    $Mmucl(pref,term,palette)
     }
 }
 
@@ -1070,8 +1075,7 @@ proc mmucl::term_pref_dialog {} {
     
     lappend gcs $gc_fg $gc_bg 
 
-    term_pref_color_gcs $gcs $Mmucl(pref,term,red) $Mmucl(pref,term,green) \
-	    $Mmucl(pref,term,blue)
+    term_pref_color_gcs $gcs
 
     set w [gtk::button new_with_label "Reset"]
     bind $w clicked [list term_pref_reset_color $gcs]
@@ -1093,24 +1097,32 @@ proc mmucl::term_pref_dialog {} {
     set r1 [g::new GtkRadioButton label Normal]
     set r2 [g::new GtkRadioButton label Image group $r1]
     set r3 [g::new GtkRadioButton label Transparent group $r1]
-    set sep [gtk::hseparator new]
-    set c1 [g::new GtkCheckButton label "Scroll background"]
-    set c2 [g::new GtkCheckButton label "Shade background"]
+    set sep [g::new GtkHSeparator]
 
+    g::object set $r1 active [expr {$Mmucl(pref,bg,type) eq "normal"}]
+    g::object set $r2 active [expr {$Mmucl(pref,bg,type) eq "image"}]
+    g::object set $r3 active [expr {$Mmucl(pref,bg,type) eq "transparent"}]
+
+    set sat_label [gtk::label new "Shade transparent or image background"]
+    set sat [gtk::hscale new_with_range 0.0 1.0 0.05]
+    gtk::range set_value $sat $Mmucl(pref,bg,saturation)
+    gtk::widget set_sensitive $sat [expr {$Mmucl(pref,bg,type) ne "normal"}]
 
     set w [gnome::pixmap_entry new id "Background image" false]
-    gtk::widget set_sensitive $w [expr {$Mmucl(pref,bg,type) eq "pixmap"}]
+    gnome::file_entry set_filename $w $Mmucl(pref,bg,image)
 
-    set script [list term_pref_update_bgtype $w $r1 $r2 $r3 $c1 $c2]
+    gtk::widget set_sensitive $w [expr {$Mmucl(pref,bg,type) eq "iamge"}]
+
+    set script [list term_pref_update_bgtype $w $r1 $r2 $r3 $sat]
+    
     bind $r1 toggled $script
-    #bind $r2 toggled $script
+    bind $r2 toggled $script
     bind $r3 toggled $script
-    bind $c1 toggled $script
-    bind $c2 toggled $script
-    bind [gnome::file_entry gtk_entry $w] activate $script
+    bind $sat value-changed $script
+    bind $w activate $script
 
     gtk::container add $frame [vbox {border-width 10 spacing 2} \
-	    $r1 $r2 $w $r3 $sep $c1 $c2]
+				 $r1 $r2 $w $r3 $sep [hbox {} $sat_label] $sat]
     gtk::box pack_start_defaults $vbox $frame
 
     set hbox [gtk::hbox new 0 10]
@@ -1177,20 +1189,21 @@ proc mmucl::session_tab_add {} {
     set Mmucl(ses,$ses,user_status) ""
     set Mmucl(ses,$ses,connect_duration) ""
 
-    set term [set Mmucl(ses,$ses,term) [g::new ZvtTerm]]
+    set term [set Mmucl(ses,$ses,term) [vte::terminal new]]
 
-    gtk::zvt_term set_font $term $Mmucl(pref,font_name) \
-	    $Mmucl(pref,use_bold) 
+    vte::terminal set_font_from_string $term $Mmucl(pref,font_name)
+    vte::terminal set_allow_bold $term $Mmucl(pref,use_bold)
+    vte::terminal set_scrollback_lines $term $Mmucl(pref,scrollback)
+    vte::terminal set_cursor_blinks $term false
+    vte::terminal set_scroll_on_output $term false
 
-    gtk::zvt_term set_blink $term 0   
-    gtk::zvt_term set_scrollback $term $Mmucl(pref,scrollback)
-
-    gtk::zvt_term set_background $term \
-	    $Mmucl(pref,bg,pixmap) $Mmucl(pref,bg,trans) $Mmucl(pref,bg,flags)
+    mmucl::term_set_background $Mmucl(ses,$ses,term) \
+	$Mmucl(pref,bg,image) $Mmucl(pref,bg,transparent) \
+	$Mmucl(pref,bg,saturation)
 
     set hbox [g::new GtkHBox]
 
-    set Mmucl(ses,$ses,termadj) [gtk::zvt_term get_adjustment $term]
+    set Mmucl(ses,$ses,termadj) [vte::terminal get_adjustment $term]
     set sb [gtk::vscrollbar new $Mmucl(ses,$ses,termadj)]
 
     set l [g::new GtkLabel label "<b>$Mmucl(cur)</b>:" use-markup 1]
@@ -1211,8 +1224,10 @@ proc mmucl::session_tab_add {} {
     gtk::notebook set_show_tabs $Mmucl(notebook) 1
 
     gtk::widget realize $term
-    gtk::zvt_term set_color_scheme $term  $Mmucl(pref,term,red) \
-	    $Mmucl(pref,term,green) $Mmucl(pref,term,blue)
+
+    vte::terminal set_colors $term \
+	$Mmucl(pref,term,fg) $Mmucl(pref,term,bg) \
+	$Mmucl(pref,term,palette)
 
     gtk::widget grab_focus $Mmucl(input)
     session_status $Mmucl(cur) disconnected
@@ -1334,6 +1349,8 @@ proc mmucl::save_config {} {
     gtk::accel_map save [file join $config(rc_dir) .accelmap.gnome]
 }
 
+# FIXME: scrollback
+
 proc mmucl::interface_init {} {
     global config
     variable Mmucl
@@ -1342,35 +1359,40 @@ proc mmucl::interface_init {} {
 	pref,use_bold       1
 	pref,font_name      "courier medium 12"
 	pref,bg,type         normal
-	pref,bg,pixmap      ""    
-	pref,bg,trans       0
-	pref,bg,flags       0  
+	pref,bg,image       ""
+	pref,bg,transparent false
+	pref,bg,saturation  0.5
 	pref,view,toolbar   1      
 	pref,view,menubar   1      
 	pref,view,statusbar 1
 	pref,scrollback     500
 	hist_cur        ""
 	current_key        ""
-	term,red_default {
-	    0x0000 0xaaaa 0x0000 0xaaaa 0x0000 0xaaaa
-	    0x0000 0xaaaa 0x5555 0xffff 0x5555 0xffff
-	    0x5555 0xffff 0x5555 0xffff 0xaaaa  0x0
-	}
-	term,green_default {
-	    0x0000 0x0000 0xaaaa 0x5555 0x0000 0x0000
-	    0xaaaa 0xaaaa 0x5555 0x5555 0xffff 0xffff
-	    0x5555 0x5555 0xffff 0xffff 0xaaaa 0x0
-	}
-	term,blue_default {
-	    0x0000 0x0000 0x0000 0x0000 0xaaaa 0xaaaa
-	    0xaaaa 0xaaaa 0x5555 0x5555 0x5555 0x5555
-	    0xffff 0xffff 0xffff 0xffff 0xaaaa 0x0
+	term,palette {
+	    {0x0000 0x0000 0x0000} 
+	    {0xaaaa 0x0000 0x0000}
+	    {0x0000 0xaaaa 0x0000} 
+	    {0xaaaa 0x5555 0x0000}
+	    {0x0000 0x0000 0xaaaa}
+	    {0xaaaa 0x0000 0xaaaa}
+	    {0x0000 0xaaaa 0xaaaa} 
+	    {0xaaaa 0xaaaa 0xaaaa}
+	    {0x5555 0x5555 0x5555}
+	    {0xffff 0x5555 0x5555}
+	    {0x5555 0xffff 0x5555}
+	    {0xffff 0xffff 0x5555}
+	    {0x5555 0x5555 0xffff}
+	    {0xffff 0x5555 0xffff}
+	    {0x5555 0xffff 0xffff}
+	    {0xffff 0xffff 0xffff}
 	}
+	term,fg {0xaaaa 0xaaaa 0xaaaa}
+	term,bg {0x0000 0x0000 0x0000}
     }
     
-    set Mmucl(pref,term,red) $Mmucl(term,red_default)
-    set Mmucl(pref,term,green) $Mmucl(term,green_default)
-    set Mmucl(pref,term,blue) $Mmucl(term,blue_default)
+    set Mmucl(pref,term,palette) $Mmucl(term,palette)
+    set Mmucl(pref,term,fg) $Mmucl(term,fg)
+    set Mmucl(pref,term,bg) $Mmucl(term,bg)
 
     set rc [file join $config(rc_dir) .pref.$config(interface)]
     if {[file exists $rc] && [catch {source $rc} err]} {
@@ -1413,7 +1435,7 @@ proc mmucl::interface_init {} {
 
     set menu {
 	stock  {gtk-copy \
-		{gtk::zvt_term copy_clipboard \
+		{vte::terminal copy_clipboard \
 		$mmucl::Mmucl(ses,$mmucl::Mmucl(cur),term)}}
 	stock  {gtk-paste \
 		{gtk::editable paste_clipboard $mmucl::Mmucl(input)}}
