@@ -18,13 +18,18 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #########################################################################
 
-# display text widget - .display$ses
-# display scrollbar - .vscroll$ses
-# input text widget - .input
-# menubar - .menubar
-# session buttons - .ses.$ses
-# bindtag class for handling MCkey - mmucl$ses 
-# .status status bar
+package require Tk
+
+# display text widget  .display$ses
+# display scrollbar    .vscroll$ses
+# input text widget    .input
+# menubar              .menubar
+# session buttons      .ses.$ses
+# status bar            .status
+# MCkey bindtag class   mmucl$ses 
+
+
+# Renders ANSI text from a session to that sessions display
 
 proc mmucl::display {ses str} {
     variable Mmucl
@@ -83,11 +88,16 @@ proc mmucl::event {ses event args} {
 	} connect {
 	    session_status $ses "connected: $host"
 	    session_button_color $ses green4
+	    update_connect_state
+
 	    set Mmucl(ses,$ses,connect_time) [clock seconds]
 	    after 60000 mmucl::update_connect_time $ses
 	} disconnect {
-	    session_button_color $ses red3
 	    session_status $ses disconnected
+	    session_button_color $ses red3
+	    update_connect_state
+
+	    .toolbar.con configure -state normal
 	} timeout {
 	    session_button_color $ses red3
 	    session_status $ses disconnected
@@ -98,12 +108,12 @@ proc mmucl::event {ses event args} {
 	    session_switched $old_ses
 	} ses_switch {
 	    lassign {old_ses} $args
+	    set ses $Mmucl(cur)
 
 	    session_switched $old_ses
-	    session_status $Mmucl(cur) $Mmucl(ses,$ses,status)
 
 	    if {$host ne ""} {
-		if {$Mmucl(ses,$Mmucl(cur),connect)} {
+		if {$Mmucl(ses,$ses,connect)} {
 		    session_button_color $ses green4
 		} else {
 		    session_button_color $ses red3
@@ -184,7 +194,7 @@ proc mmucl::session_switched {old_ses} {
 
     .ses.$old_ses configure -relief sunken
     .ses.$ses configure -relief raised
-    
+
     grid remove .display$old_ses
     grid remove .vscroll$old_ses
     grid .display$ses
@@ -192,6 +202,30 @@ proc mmucl::session_switched {old_ses} {
 
     bindtags $Mmucl(entry) \
 	    [lreplace [bindtags $Mmucl(entry)] 0 0 mmucl$ses]
+
+    session_status $ses $Mmucl(ses,$ses,status)
+
+    update_connect_state
+}
+
+# Update user interface elements about connect state change
+
+proc mmucl::update_connect_state {} {
+    variable Mmucl
+
+    if {$Mmucl(ses,$Mmucl(cur),connect)} {
+	.toolbar.con configure -state disabled
+	.toolbar.send configure -state normal
+	.menubar.mmucl entryconfigure 0 -state disabled
+	.menubar.mmucl entryconfigure 1 -state disabled
+	.menubar.mmucl entryconfigure 4 -state normal
+    } else {
+	.toolbar.con configure -state normal
+	.toolbar.send configure -state disabled
+	.menubar.mmucl entryconfigure 0 -state normal
+	.menubar.mmucl entryconfigure 1 -state normal
+	.menubar.mmucl entryconfigure 4 -state disabled
+    }
 }
 
 proc mmucl::update_connect_time {ses} {
@@ -209,7 +243,7 @@ proc mmucl::update_connect_time {ses} {
 
 proc balloon_add {w message} {
     bind $w <Enter> [list after 500 balloon_draw $w [list $message]]
-    bind $w <Leave> [namespace code [list balloon_erase $w $message]]\
+    bind $w <Leave> [list balloon_erase $w $message]
 }
 
 proc balloon_draw {w message} {
@@ -304,12 +338,21 @@ proc mmucl::user_entry_scroll_down {w} {
     return
 }
 
-proc mmucl::tk_script {script descript} {
+# Evaluate a script and on error popup a message box.
+# Optionally print the stack trace.
+
+proc mmucl::tk_script {script descript {print_stack_trace 1}} {
     global errorInfo
     variable Mmucl
 
     if {[catch $script error]} {
-	report $Mmucl(cur) error "$error\nStack trace:\n$::errorInfo"
+	set msg $error
+	
+	if {$print_stack_trace} {
+	    append msg "\nStack trace:\n" $::errorInfo
+	}
+
+ 	report $Mmucl(cur) error 
 	tk_messageBox -icon error -message "Error: $descript\n$error" -type ok
 	return 0
     }
@@ -317,7 +360,7 @@ proc mmucl::tk_script {script descript} {
     return 1
 }
 
-# ugly
+# FIXME: ugly, redo
 proc mmucl::tk_connect {} {
     variable Mmucl
     
@@ -329,7 +372,8 @@ proc mmucl::tk_connect {} {
     }
     
     toplevel .con
-    wm title .con Connect
+    wm title .con "Mmucl - Connect"
+    wm transient .con .
 
     frame .con.f -relief groove -bd 2
     grid [label .con.f.hostl -text Host:]
@@ -387,10 +431,10 @@ proc mmucl::tk_connect_do {} {
     variable Mmucl
 
     if {![string equal $Mmucl(tmpchar) ""]} {
-	set res [tk_script [list MCchar $Mmucl(cur) load $Mmucl(tmpchar)] "Loading char"]
+	set res [tk_script [list MCchar $Mmucl(cur) load $Mmucl(tmpchar)] "Loading char" 0]
     } else {
 	set res [tk_script [list MCconnect $Mmucl(cur) [.con.f.host get]\
-		[.con.f.port get]] "Connecting to host"]
+		[.con.f.port get]] "Connecting to host" 0]
     }
     
     if {$res} {
@@ -433,14 +477,14 @@ proc mmucl::tk_edit {proc} {
 
     toplevel $w
     wm title $w "Mmucl - $title"
-    wm group $w .
     wm transient $w .
 
     form $w.form $labels MC$proc $list
     pack $w.form -side top -pady 3
  
     frame $w.b
-    pack [button $w.b.help -text Help -command [list mmucl::MChelp $Mmucl(cur) $proc]]\
+    pack [button $w.b.help -text Help \
+	      -command [list mmucl::MChelp $Mmucl(cur) $proc]]\
 	    [button $w.b.exit -text Close -command [list destroy $w]] \
 	    -side left
     pack $w.b -side bottom
@@ -454,19 +498,21 @@ proc mmucl::tk_edit {proc} {
 
 # evaluate a script bound to a key
 
-proc mmucl::key_eval {ses group key cur_key} {
+proc mmucl::key_eval {id cur_key} {
     variable Mmucl
 
+    set ses $Mmucl(cur)
+    set x ses,$ses,group
     set Mmucl(ses,$ses,current_key) $cur_key
-    set script [group_member_get $ses $group key $key]
-    set code [catch {$ses eval $script} error]
-
-    if {$code} {
-	if {$code == 3} {
-	    return -code break
-	}
+    
+    foreach group $Mmucl($x,forkey,key,$id) {
+	set code [catch {$ses eval $Mmucl($x,val,$group,key,$id)} error]
 	
-	report $ses error "key $key: $error"
+	if {$code == 1} {
+	    report $ses error "key $id: $error"
+	} else {
+	    return -code $code
+	}
     }
 }
 
@@ -481,11 +527,9 @@ proc mmucl::tk_key_event {id} {
     return $event
 }
 
-
 proc mmucl::key_set_hook {ses group key val arglist} {
     key_validate $key
-    bind mmucl$ses [tk_key_event $key] \
-	[list mmucl::key_eval $ses $group $key %K]
+    bind mmucl$ses [tk_key_event $key] [list mmucl::key_eval $key %K]
 }
 
 proc mmucl::key_delete_hook {ses group key} {
@@ -493,16 +537,21 @@ proc mmucl::key_delete_hook {ses group key} {
 }
 
 
-proc mmucl::MChelp {cur {subject ""}} {
+proc mmucl::MChelp {ses args} {
     global config ntki_embed
 
+    set syntax {{? node ""}}
+    check help $syntax $args
+
     if {![info exists ntki_embed]} {
 	set ntki_embed 1
 	uplevel #0 [list source [file join $config(lib_dir) lib ntkinfo.tcl]]
 	ntki::init
     }
     
-    ntki::win (mmucl)$subject
+    ntki::win (mmucl)$arg(node)
+
+    return
 }
 
 # Modify the command line.
@@ -559,31 +608,30 @@ proc mmucl::MCcline {cur args} {
 proc mmucl::tk_source {} {
     variable Mmucl
 
-    set file [tk_getOpenFile -title "Load a file"]
+    set file [tk_getOpenFile -title "Mmucl - Load a file"]
     
     if {$file ne ""} {
-	tk_script [list $Mmucl(cur) invokehidden source $file] \
-		"Loading script"
+	tk_script [list $Mmucl(cur) invokehidden source $file] "Loading script"
     }
 }
 
 proc mmucl::tk_textin {} {
     variable Mmucl
 
-    set file [tk_getOpenFile -title "Send a file"]
+    set file [tk_getOpenFile -title "Mmmucl - Send a file"]
     
     if {$file ne ""} {
-	tk_script [list MCtextin $Mmucl(cur) $file] "Sending file"
+	tk_script [list MCtextin $Mmucl(cur) $file] "Sending file" 0
     }
 }
 
 proc mmucl::tk_dump {} {
     variable Mmucl
 
-    set file [tk_getSaveFile -title "Save state to file"]
+    set file [tk_getSaveFile -title "Mmucl - Save state to file"]
    
     if {$file ne ""} {
-	tk_script [list MCdump $Mmucl(cur) -all -- $file] "Saving state"
+	tk_script [list MCdump $Mmucl(cur) -all -- $file] "Saving state" 0
     }
 }
 
@@ -670,7 +718,7 @@ proc mmucl::form_add {w proc list} {
     }
 
     # assumes MCname procs...
-    tk_script $script "Adding [string range $proc 2 end]"
+    tk_script $script "Adding [string range $proc 2 end]" 0
     form_listbox_update $w $proc
 }
 
@@ -729,6 +777,19 @@ proc mmucl::form_set_entries {w list} {
     }
 }
 
+# Reset preferences to default
+
+proc mmucl::default_pref {} {
+    variable Mmucl
+
+    array set Mmucl $Mmucl(pref_defaults)
+
+    font_changed
+    
+    foreach ses $Mmucl(sessions) {
+	session_set_color $ses
+    }
+}
 
 proc mmucl::font_changed {} {
     variable Mmucl
@@ -749,13 +810,6 @@ proc mmucl::tk_ses_new {} {
     MCsession $mmucl::Mmucl(cur) new
 }
 
-proc mmucl::tk_ses_close {} {
-    variable Mmucl
-
-    tk_script {MCsession $Mmucl(cur) close $Mmucl(cur)} \
-	    "Closing session $mmucl::Mmucl(cur)"
-}
-
 proc mmucl::font_family_menu {w} {
     variable Mmucl
 
@@ -769,6 +823,17 @@ proc mmucl::font_family_menu {w} {
     }
 }
 
+proc mmucl::mmucl_char_menu {w} {
+    variable Mmucl
+
+    $w delete 0 end
+
+    foreach char [MCchar $Mmucl(cur) names] {
+	$w add command -label $char \
+	    -command [list mmucl::MCchar $Mmucl(cur) load $char]
+    }
+}
+
 proc mmucl::ses_name {ses} {
     variable Mmucl
 
@@ -834,7 +899,7 @@ proc mmucl::choose_color {title which w entry {tag ""}} {
 }
 
 
-proc mmucl::tk_session_close {} {
+proc mmucl::tk_ses_close {} {
     variable Mmucl
 
     set ses $Mmucl(cur)
@@ -893,6 +958,33 @@ proc mmucl::tk_exit {} {
     MCexit $Mmucl(cur)
 }
 
+# Set the display colors for the session
+
+proc mmucl::session_set_color {ses} {
+    variable Mmucl
+
+    set w .display$ses
+
+    # default colors
+    $w configure -foreground $Mmucl(pref,foreground) \
+	-background $Mmucl(pref,background)
+    
+    # FIXME: should reverse fg and bg
+    $w tag configure 7 -background [$w cget -foreground] \
+	-foreground [$w cget -background]
+    
+    foreach tag {30 31 32 33 34 35 36 37} {
+	$w tag configure $tag -foreground $Mmucl(pref,ansi,$tag)
+    }
+    
+    foreach tag {40 41 42 43 44 45 46 47} {
+	$w tag configure $tag -background $Mmucl(pref,ansi,$tag)
+    }
+    
+    # needed to make selection visible.
+    $w tag raise sel    
+}
+
 proc mmucl::session_added {} {
     variable Mmucl
 
@@ -902,9 +994,7 @@ proc mmucl::session_added {} {
     set Mmucl(ses,$ses,connect_duration) ""
 
     text .display$ses -state disabled -yscrollcommand [list .vscroll$ses set] \
-	    -font [list $Mmucl(pref,font,family) $Mmucl(pref,font,size)] \
-	    -foreground $Mmucl(pref,foreground) \
-	    -background $Mmucl(pref,background)
+	    -font [list $Mmucl(pref,font,family) $Mmucl(pref,font,size)]
     scrollbar .vscroll$ses -orient vertical -command [list .display$ses yview]
 
     set w .display$ses
@@ -925,20 +1015,7 @@ proc mmucl::session_added {} {
     # should blink
     $w tag configure 5 -underline 1
     
-    # FIXME: should reverse fg and bg
-    $w tag configure 7 -background [$w cget -foreground] \
-	    -foreground [$w cget -background]
-    
-    foreach tag {30 31 32 33 34 35 36 37} {
-	$w tag configure $tag -foreground $Mmucl(pref,ansi,$tag)
-    }
-
-    foreach tag {40 41 42 43 44 45 46 47} {
-	$w tag configure $tag -background $Mmucl(pref,ansi,$tag)
-    }
-    
-    # needed to make selection visible.
-    $w tag raise sel    
+    session_set_color $ses
 
     pack [button .ses.$ses -text [ses_name $ses] \
 	    -command "mmucl::MCsession \$mmucl::Mmucl(cur) switch $ses"] \
@@ -958,10 +1035,7 @@ proc mmucl::interface_init {} {
     variable Mmucl
     global config tcl_platform auto_path
 
-    array set Mmucl {
-	ansi_tags {{} 37 40}
-	rxp,ansi_parse {^\[([0-9;]*)m(.*)$}
-	hist_cur        ""
+    set Mmucl(pref_defaults) {
 	pref,font,family Courier
 	pref,font,size   12
 	pref,ansi,30 slategrey   
@@ -986,42 +1060,20 @@ proc mmucl::interface_init {} {
 	pref,view,menubar   1      
 	pref,view,statusbar 1
     }
+
+    array set Mmucl {
+	ansi_tags {{} 37 40}
+	rxp,ansi_parse {^\[([0-9;]*)m(.*)$}
+	hist_cur        ""
+    }
+
+    array set Mmucl $Mmucl(pref_defaults)
     
     set rc [file join $config(rc_dir) .pref.$config(interface)]
     if {[file exists $rc] && [catch {source $rc} err]} {
 	puts stderr "BUG: loading $rc: $err"
     }
     
-    # It's pathetic that this isn't supported!
-
-    if {[catch {package require Tk 8.4}]} {
-	# try both libtk8.4 and libtk84
-	
-	if {[lsearch -exact [package names] Tk] == -1} {
-	    set version [info tclversion]
-	    set libs [glob -nocomplain \
-			  [file join [file dirname [info library]] \
-			       libtk${version}[info sharedlibextension]*]]
-	    
-	    if {[llength $libs]} {
-		load [lindex $libs 0] Tk
-	    } else {
-		regsub -- {\.} $version "" version
-		
-		set libs [glob -nocomplain \
-			      [file join [file dirname [info library]] \
-				   libtk${version}[info sharedlibextension]*]]
-		
-		if {[llength $libs]} {
-		    load [lindex $libs 0] Tk
-		} else {
-		    error "Cannot find Tk shared lib."
-		}
-	    }
-	}
-    
-    }
-
     option add *Menu*tearOff 0
 
     foreach img [glob [file join $config(lib_dir) images *.*]] {
@@ -1055,6 +1107,7 @@ proc mmucl::interface_init {} {
     set w [menu .menubar.mmucl]
     
     $w add command -label Connect -underline 0 -command mmucl::tk_connect
+    $w add cascade -label "Load Character" -underline 1 -menu $w.char
     $w add separator
     $w add command -label Load -underline 0 -command mmucl::tk_source
     $w add command -label Send -underline 0 -command mmucl::tk_textin
@@ -1063,6 +1116,9 @@ proc mmucl::interface_init {} {
     $w add command -label Quit -accelerator Ctrl+Q -underline 0 \
 	    -command mmucl::tk_exit
 
+    set w [menu  .menubar.mmucl.char]
+    bind $w <FocusIn> {mmucl::mmucl_char_menu %W}
+
     bind . <Control-q> {mmucl::tk_exit}
 
     set w [menu .menubar.edit]
@@ -1098,22 +1154,19 @@ proc mmucl::interface_init {} {
 
     $w add command -label New -underline 0 -accelerator Ctrl+N \
 	    -command {mmucl::tk_ses_new}
-    #$w add cascade -label Current -underline 1 -menu $w.current
     $w add cascade -label Snoop -underline 0 -menu $w.snoop
     $w add command -label Close -underline 0 -accelerator Ctrl+W \
 	    -command {mmucl::tk_ses_close}
 
-    bind . <Control-n> {mmucl::MCsession $mmucl::Mmucl(cur) new}
-    bind . <Control-w> [namespace code tk_session_close]
+    bind . <Control-n> [namespace code {MCsession $Mmucl(cur) new}]
+    bind . <Control-w> [namespace code {tk_ses_close}]
     
-    #set w [menu  .menubar.ses.current]
-    #bind $w <FocusIn> {mmucl::ses_cur_menu %W}
-
     set w [menu  .menubar.ses.snoop]
     bind $w <FocusIn> {mmucl::ses_snoop_menu %W}
 
     set w [menu .menubar.set]
     
+    $w add command -label "Default" -underline 0 -command {mmucl::default_pref}
     $w add cascade -label "Font family" -menu $w.font_fam -underline 0
     $w add cascade -label "Font size" -menu $w.font_size -underline 5
     $w add cascade -label "Foreground color" -menu $w.fg_color -underline 1
@@ -1252,6 +1305,9 @@ proc mmucl::interface_init {} {
 
     # FIXME
     wm command . $::argv
+    
+    # FIXME
+    update_connect_state
 
     rename interface_init ""
 }
