@@ -4,7 +4,7 @@ proc cb {data ev args} {
     puts $ev
     puts $args
     
-    connect_cancel [lindex $args 0]
+#    connect_cancel [lindex $args 0]
 }
 
 proc bgerror {msg} {
@@ -25,7 +25,7 @@ proc whee {} {
 }
 
 whee
-connect yahoo.com 3000 cb "" -max_attempts 3 -timeout 3000 -thread 1
+connect red.cn 3000 cb "" -max_attempts 3 -timeout 3 -attempt_wait 5
 
 vwait forever
  
