@@ -57,7 +57,7 @@ proc ltrimleft {list n} {
 proc duration {secs} {
     set res ""
 
-    foreach div {86400 3600 60} mod {0 24 60} name {day hr min} {
+    foreach div {86400 3600 60} mod {0 24 60} name {day hour minute} {
         set n [expr {$secs / $div}]
 
         if {$mod > 0} {
@@ -65,9 +65,9 @@ proc duration {secs} {
 	}
 
         if {$n > 1} {
-            append res $n " " $name s
+            append res $n " " $name "s "
 	} elseif {$n == 1} {
-            append res $n " " $name
+            append res $n " " $name " "
 	}
     }
 
