@@ -25,7 +25,7 @@ source [file join $config(lib_dir) lib connect.tcl]
 
 # All user scripts are evaluated in a seperate interp. The interp name
 # is $ses where $ses is a session id. In theory the Mmucl array
-# shouldn't be examined interfaces. Instead the various MCname commands
+# shouldn't be examined by interfaces. Instead the various MCname commands
 # should be used to manipulate it. Practice is somewhat different. :)
 #
 # $Mmucl(cur) is the current session.
@@ -84,7 +84,7 @@ proc mmucl::init {} {
 	bg_white 47 bg_default 49
     }
 
-    array set Mmucl {
+    set Mmucl(default_cfg) {
 	cfg,reconnect      1
 	cfg,actions        1
 	cfg,subs           1
@@ -104,6 +104,9 @@ proc mmucl::init {} {
 	cfg,split_char     \;
 	cfg,use_threads    1
 	cfg,use_mccp       1
+    }
+
+    array set Mmucl {
 	chars              {}
 	history            {}
 	rxp,buf            {^(.*\n)(.*)$}
@@ -113,6 +116,8 @@ proc mmucl::init {} {
 	types              {alias action sub key}
     }
 
+    array set Mmucl $Mmucl(default_cfg)
+
     set Mmucl(save) [array names Mmucl cfg,*]
     lappend Mmucl(save) chars history
 
@@ -124,7 +129,7 @@ proc mmucl::init {} {
     set Mmucl(hist_loc) [llength $Mmucl(history)]
 
     set Mmucl(has,MCCP) [expr {![catch {package require MCCP 0.06}]}]
-    set Mmucl(has,Thread) [expr {![catch {package require Thread}]}]
+    set Mmucl(has,Thread) [expr {![catch {package require Thread 2.5}]}]
 
     if {$Mmucl(has,Thread)} {
 	thread::errorproc mmucl::thread_error
@@ -191,13 +196,13 @@ proc mmucl::ses_new {} {
     if {$ses == 0} {
 	interface_init
 	
-	print $ses "** Mmucl $config(version)"
+	report $ses msg "Mmucl $config(version)"
 
 	if {$Mmucl(has,Thread)} {
-	    print $ses "** Thread extension loaded."
+	    report $ses msg "Thread extension loaded."
 	}
 	if {$Mmucl(has,MCCP)} {
-	    print $ses "** MCCP extension loaded."
+	    report $ses msg "MCCP extension loaded."
 	}
 
 	print $ses ""
@@ -216,8 +221,9 @@ proc mmucl::ses_new {} {
     if {[file exists $rc]} {
 	source_user $ses $rc
     } else {
-	print $ses "$rc does not exist. Using default."
-	source_user $ses [file join $config(lib_dir) examples mmucl.rc]
+	set default [file join $config(lib_dir) script mmucl.rc]
+	report $ses msg "$rc does not exist. Using default."
+	source_user $ses $default
     }
 
     return $ses
@@ -322,8 +328,12 @@ proc mmucl::MCsession {ses args} {
     return
 }
 
-proc mmucl::MCdebug {ses script} {
-    return [namespace eval ::mmucl $script]
+proc mmucl::MCdebug {ses args} {
+    return [namespace eval ::mmucl:: [concat $args]]
+}
+
+proc mmucl::MCdebugsource {ses file} {
+    uplevel \#0 source $file
 }
 
 # Groups aggregate and isolate aliases, actions, keys and subs.
@@ -511,9 +521,8 @@ proc mmucl::MCstatus {ses args} {}
 proc mmucl::MCclear {ses args} {}
 proc mmucl::save_config {} {}
 
-# Called when there is mud output.
-# It buffers an incomplete line and flushes it
-# if read_mud is not called again soon.
+# Called when there is mud output.Buffers an incomplete line and flushes
+# the line if read_mud is not called again soon.
 
 proc mmucl::read_mud {ses} {
     variable Mmucl
@@ -593,12 +602,12 @@ proc mmucl::match_actions {ses str} {
     }
 }
 
-proc mmucl::meta_display {ses str} {
+proc ::mmucl::meta_display {ses str} {
     variable Mmucl
 
     if {$Mmucl(ses,$ses,snoop) && $ses != $Mmucl(cur)} {
 	foreach line [split $str \n] {
-	    display $Mmucl(cur) [color [list "** $ses:" bold $line \n]]
+	    display $Mmucl(cur) [color "** $ses:" bold]$line\n
 	}
     }
 
@@ -611,6 +620,7 @@ proc mmucl::report {ses event args} {
     variable Mmucl
     
     set color $Mmucl(cfg,report_color)
+
     switch -exact -- $event {
 	error {
 	    set color $Mmucl(cfg,error_color)
@@ -633,6 +643,8 @@ proc mmucl::report {ses event args} {
 	} load {
 	    lassign {file} $args
 	    set msg "Loading $file..."
+	} msg {
+	    set msg [lindex $args 0]
 	} default {
 	    error "no handler for $event"
 	}
@@ -1020,8 +1032,7 @@ proc mmucl::MCconnect {ses args} {
 	    
     connect $arg(host) $arg(port) mmucl::connect_callback $ses \
 	    -max_attempts [expr {$Mmucl(cfg,reconnect) + 1}]   \
-	    -timeout [expr {$Mmucl(cfg,timeout) * 1000}]       \
-	    -thread $Mmucl(cfg,use_threads)
+	    -timeout $Mmucl(cfg,timeout) -thread $Mmucl(cfg,use_threads)
     return
 }
 
@@ -1071,6 +1082,7 @@ proc mmucl::MCconfig {ses args} {
 	names  {{? pattern *}}
 	print  {{? pattern *}}
 	state  {}
+	reset {}
     }
 
     switch -exact -- [check config $syntax $args 1] {
@@ -1108,6 +1120,8 @@ proc mmucl::MCconfig {ses args} {
 	    }
 
 	    return $res
+	} reset {
+	    array set Mmucl $Mmucl(default_cfg)
 	}
     }
     
@@ -1765,9 +1779,7 @@ proc bgerror {msg} {
 proc mmucl::MCbgerror {ses msg} {
     global errorInfo
 
-    report $ses error "user background error: $msg"
-    print "Stack trace:"
-    print $errorInfo
+    report $ses error "Background error in user script: $msg\nStack trace:\n$::errorInfo"
 }
 
 proc mmucl::thread_error {id msg} {
